﻿// Module: Ports.js
//  This module is used to manage the settings of the ports

// Dependencies, Constants and Globals
//#region Dependencies, Constants and Globals

// Dependencies
//#region Dependencies

// External dependencies
var moment = require('moment');
var fs = require('fs');
var cp = require('child_process');
var path = require('path');

// Peak (global) dependencies (set during Initialization)
var Logger;
var SocketIo;
var NetSnmp;
var SmtpManager;
var SnmpHostSessionsArray = [];// Instance array for list of SNMP host sessions

//#endregion Dependencies

// Constants
//#region Constants
var PEAK_PORTS = 4;
var PEAK_PORTS_SETTINGS = "./data/ports.json";
var PEAK_PORTS_LOGFILE = "./log/port";
var PEAK_DEVICE_SETTINGS = "./data/device"
var PEAK_NO_DEVICE = "NONE";
var PEAK_NO_DEVICE_GROUP = "NONE";

//#endregion Constants

// Globals
//#region Globals

// The following array holds, in memory, the current configuration of the Peak's ports
var PortSettingsArray = {};

// Default Ports Array if ports.json is not readable
var DefaultPortSettingsArray = [
    // Port#,       Port Name,         Device Name,      Device Group,          Enabled,          TimeChanged,       Device object,        Device Info
    { "Port": "0", "Name": "Port 1", "Device": "NONE", "DeviceGroup": "NONE", "Enabled": false, "TimeChanged": 0, "DeviceObject": null, "DeviceModel": null },
    { "Port": "1", "Name": "Port 2", "Device": "NONE", "DeviceGroup": "NONE", "Enabled": false, "TimeChanged": 0, "DeviceObject": null, "DeviceModel": null },
    { "Port": "2", "Name": "Port 3", "Device": "NONE", "DeviceGroup": "NONE", "Enabled": false, "TimeChanged": 0, "DeviceObject": null, "DeviceModel": null },
    { "Port": "3", "Name": "Port 4", "Device": "NONE", "DeviceGroup": "NONE", "Enabled": false, "TimeChanged": 0, "DeviceObject": null, "DeviceModel": null }
];

// List of supported devices
var DeviceModelsArray = [];

// List of device groups
var DeviceGroupsArray = ["Access", "Environment", "Power", "RF"];

//#endregion Globals

//#endregion Dependencies, Constants and Globals

//#region Functions

//#region General Functions
// Initialize the devices upon system start
//      - Checks if database table exists, if not creates the table and fills it with default data for each device
//      - Else reads from the database and updates PortSettingsArray
module.exports.Initialize = Initialize;
function Initialize(logger, io, smtpManager, netSnmp, callback) {

    // Initialize the dependencies and globals
    //#region Initialize the dependencies and globals
    Logger = logger;
    SocketIo = io;
    SmtpManager = smtpManager;
    NetSnmp = netSnmp;
    Logger.debug("Ports.Initialize: Initialized dependencies and globals");

    callback();
    //#endregion Initialize the dependencies and globals

}

// Initialize the snmpHostSessionsArray used for send SNMP messages
module.exports.SetSnmpHostSessionsArray = SetSnmpHostSessionsArray;
function SetSnmpHostSessionsArray(snmpHostSessions, callback) {
    SnmpHostSessionsArray = snmpHostSessions.slice(0);// Copy array

    // Update each device
    for (var i = 0; i < PEAK_PORTS; i++) {
        if (PortSettingsArray[i].Device !== "NONE" && PortSettingsArray[i].DeviceObject) {
            // If the port has a device allocated, instantiate a device object for the port
            Logger.info("Ports.SetSnmpHostSessionsArray: PortArray[" + i + "] - Updating SnmpSessionArray for device=" + PortSettingsArray[i].Device);
            PortSettingsArray[i].DeviceObject.SetSnmpHostSessionsArray(SnmpHostSessionsArray, function () { });// Set Sessions Array
        }
    }

    callback();
}

//#endregion General Functions

// KnownDeviceArray functions
//  LoadDeviceModelsArray - load list of supported devices [exported - called by Crest application]
//  GetDeviceModelsArray - retrieves DeviceList
//  GetDeviceGroupsArray - retreives DeviceGroupList
//#region DeviceModelsArray functions

// Load list of supported devices from the /devices directory
module.exports.LoadDeviceModelsArray = LoadDeviceModelsArray;
function LoadDeviceModelsArray(callback) {
    Logger.trace("Ports.LoadDeviceModelsArray: Load DeviceModelsArray ============================================================");
    fs.readdir("./devices", function (err, files) {
        var count = 0;
        for (var i = 0; i < files.length; i++) {
            if (path.extname(files[i]) !== '.json') continue;// Only load JSON files

            // Read file and store name in an array
            DeviceModelsArray[count] = require("./devices/" + files[i]);
            Logger.info("Ports.LoadDeviceModelsArray: Loading Device[" + count + "] (File=" + files[i] + "): id=" + DeviceModelsArray[count].id + " (" + DeviceModelsArray[count].title + " v" + DeviceModelsArray[count].version + " - " + DeviceModelsArray[count].date + ")");
            count++;
        }

        Logger.trace("Ports.LoadDeviceModelsArray: Loaded DeviceModelsArray ============================================================");
        callback();
    });
}

// GetDeviceModelsArray
module.exports.GetDeviceModelsArray = GetDeviceModelsArray;
function GetDeviceModelsArray(callback) {
    return DeviceModelsArray;
}

// GetDeviceGroupsArray
module.exports.GetDeviceGroupsArray = GetDeviceGroups;
function GetDeviceGroups(callback) {
    return DeviceGroupsArray;
}
//#endregion DeviceModelsArray functions

// PortSettingsArray functions
//  LoadPortSettingsArray - load ports into memory [exported - called by Crest application]
//  UpdatePortSettingsArrayPort - updates a port in PortSettingsArray and ports.json
//  UpdatePortSettingsArrayPortDevice - updates a device in the PortSettingsArray
//  GetPortSettingsArray - retreives PortSettingsArray
//  GetDeviceInfo(port) - retrieves DeviceInformation parameter from device
//#region PortSettingsArray functions

// Load the PortSettingsArray 
//  - Loads the PortSettingsArray in memory, from either default or persistent storage
module.exports.LoadPortSettingsArray = LoadPortSettingsArray;
function LoadPortSettingsArray(callback) {
    Logger.trace("Ports.LoadPortSettingsArray: Loading PortSettingsArray ============================================================");
    try {
        if (fs.existsSync(PEAK_PORTS_SETTINGS)) {
            Logger.info("Ports.LoadPortSettingsArray: Loading PortSettingsArray from '" + PEAK_PORTS_SETTINGS + "'");

            PortSettingsArray = require(PEAK_PORTS_SETTINGS);

            // Load an instance of each allocated device
            for (var i = 0; i < PEAK_PORTS; i++) {
                // If the port has a device allocated, instantiate a device object for the port
                Logger.info("Ports.LoadPortSettingsArray: PortArray[" + i + "] - Loading device=" + PortSettingsArray[i].Device);

                if (PortSettingsArray[i].Device !== "NONE") {
                    // Instantiate via constructor, which is created with default device model ...
                    require("./devices/" + PortSettingsArray[i].Device + ".js")(i, Logger, SocketIo, SmtpManager, NetSnmp, function (deviceObject) {
                        PortSettingsArray[i].DeviceObject = deviceObject;
                        PortSettingsArray[i].DeviceModel = PortSettingsArray[i].DeviceObject.GetDeviceModel();
                        Logger.info("Ports.LoadPortSettingsArray: PortArray[" + i + "] - Instantiated device=" + PortSettingsArray[i].Device + " with default model");

                        //#region Load the persistent device model settings from JSON file
                        // Load the persistent device model settings from JSON file
                        if (fs.existsSync(PEAK_DEVICE_SETTINGS + i + ".json")) {
                            var model = require(PEAK_DEVICE_SETTINGS + i + ".json");
                            Logger.info("Ports.LoadPortSettingsArray: PortArray[" + i + "] - Retrieved saved model for device=" + PortSettingsArray[i].Device);

                            UpdatePortSettingsArrayPortDevice(i, model, function (updatedmodel) {
                                Logger.info("Ports.LoadPortSettingsArray: PortArray[" + updatedmodel.port + "] - Updated device object using saved settings for '" + PortSettingsArray[updatedmodel.port].Device + "' Enabled=" + (PortSettingsArray[updatedmodel.port].Enabled ? "YES" : "NO"));
                                PortSettingsArray[updatedmodel.port].DeviceModel = updatedmodel;
                                PortSettingsArray[updatedmodel.port].DeviceObject.Enable(PortSettingsArray[updatedmodel.port].Enabled);// Start the device
                                PortSettingsArray[updatedmodel.port].DeviceObject.SendEvents(PortSettingsArray[updatedmodel.port].SendEvents);// SendEvents from the device
                                PortSettingsArray[updatedmodel.port].DeviceObject.SetSnmpHostSessionsArray(SnmpHostSessionsArray, function () { });// Set Sessions Array
                            })
                        }

                        //  Previous Device model persistence didn't exist, so create for future use ...
                        else {
                            Logger.info("Ports.LoadPortSettingsArray: PortArray[" + i + "] - Saved device model does not exist for device=" + PortSettingsArray[i].Device);

                            UpdatePortSettingsArrayPortDevice(i, PortSettingsArray[i].DeviceObject.DeviceModel, function (updatedmodel) {
                                Logger.info("Ports.LoadPortSettingsArray: PortArray[" + updatedmodel.port + "] - Updated device object using default settings for '" + PortSettingsArray[updatedmodel.port].Device + "' Enabled=" + (PortSettingsArray[updatedmodel.port].Enabled ? "YES" : "NO"));
                                PortSettingsArray[updatedmodel.port].DeviceModel = updatedmodel;
                                PortSettingsArray[updatedmodel.port].DeviceObject.Enable(PortSettingsArray[updatedmodel.port].Enabled);// Start the device
                                PortSettingsArray[updatedmodel.port].DeviceObject.SendEvents(PortSettingsArray[updatedmodel.port].SendEvents);// SendEvents from the device
                                PortSettingsArray[updatedmodel.port].DeviceObject.SetSnmpHostSessionsArray(SnmpHostSessionsArray, function () { });// Set Sessions Array
                            });
                        }
                        //#endregion Load the persistent device model settings from JSON file
                    });
                }
            }
        }

        else {
            // Port settings file does not exist, created with default settings ...
            Logger.warn("Ports.LoadPortSettingsArray: '" + PEAK_PORTS_SETTINGS + "' does not exist, creating default file ...");
            PortSettingsArray = DefaultPortSettingsArray;// Take on default values
            fs.truncate(PEAK_PORTS_SETTINGS, 0, function () {
                fs.writeFile(PEAK_PORTS_SETTINGS, JSON.stringify(DefaultPortSettingsArray), function (error) {
                    if (error) {
                        Logger.error("Ports.LoadPortSettingsArray: '" + PEAK_PORTS_SETTINGS + "' create error=" + error);
                    }
                    Logger.info("Ports.LoadPortSettingsArray: '" + PEAK_PORTS_SETTINGS + "' created with default settings=" + JSON.stringify(DefaultPortSettingsArray));
                    cp.exec("sync", function (error, stdout, stderr) {
                        Logger.debug("Ports.LoadPortSettingsArray: sync " + PEAK_PORTS_SETTINGS);
                        if (error) {
                            Logger.error("Ports.LoadPortSettingsArray: sync error=" + error);
                        } else {
                            Logger.warn("Ports.LoadPortSettingsArray: Port Settings Updated! '" + PEAK_PORTS_SETTINGS + "'");
                        }
                    });
                });
            });
        }

    } catch (error) {
        Logger.error("Ports.LoadPortSettingsArray: Catch error=" + error);
    } finally {
        Logger.trace("Ports.LoadPortSettingsArray: Loaded PortSettingsArray ============================================================");
        callback();
    }
}

// UpdatePortSettingsArrayPort
//  - Updates the PortSettingsArray based on port data sent from client
module.exports.UpdatePortSettingsArrayPort = UpdatePortSettingsArrayPort;
function UpdatePortSettingsArrayPort(newPortSettings, callback) {
    //---------------------------------------------------------------------
    //  Updates of the port have the following cases:
    //      Case #1:    new=NONE        old=NONE
    //      Case #2:    new=NONE        old=Device
    //      Case #3:    new=Device      old=NONE
    //      Case #4:    new=Device      old=Device (new!=old)
    //      Case #5:    new=Device      old=Device (new==old)

    // Set the new device and track previous device
    var _port = newPortSettings.Port;
    Logger.info("Ports.UpdatePortSettingsArrayPort: Port[" + _port + "]=" + PortSettingsArray[_port].Device + " -> " + newPortSettings.Device);

    // Case #1 & #2: New Device=PEAK_NO_DEVICE - we're clearing this Port
    //#region Case #1 & #2: New Device=PEAK_NO_DEVICE - we're clearing this Port
    if (newPortSettings.Device === PEAK_NO_DEVICE) {

        // Case #2a: If there was a different device on this newPortSettings, stop and remove it
        //#region Case #2a: If there was a different device on this newPortSettings, stop and remove it
        if (PortSettingsArray[_port].Device !== PEAK_NO_DEVICE) {
            // Stop processing in a device was previously on this port
            PortSettingsArray[_port].DeviceObject.Enable(false);// Stop the previous device
            PortSettingsArray[_port].DeviceObject.SendEvents(false);// Stop sending events from the device

            Logger.warn("Ports.UpdatePortSettingsArrayPort: Port[" + _port + "]=" + PortSettingsArray[_port].Device + " was stopped and removed!");
        }
        //#endregion Case #2a: If there was a different device on this newPortSettings, stop and remove it

        // Case #1 & #2b: De-instantiate previous device and set port to empty
        //#region Case #1 & #2b: De-instantiate previous device and set port to empty
        PortSettingsArray[_port].Name = "Port " + (Number(_port)+1);
        PortSettingsArray[_port].Device = PEAK_NO_DEVICE;
        PortSettingsArray[_port].DeviceGroup = PEAK_NO_DEVICE_GROUP;
        PortSettingsArray[_port].Enabled = false;
        PortSettingsArray[_port].SendEvents = false;
        PortSettingsArray[_port].DeviceObject = null;
        PortSettingsArray[_port].TimeChanged = moment();// Device has changed
        PortSettingsArray[_port].DeviceModel = null;

        Logger.info("Ports.UpdatePortSettingsArrayPort: Port[" + _port + "] Device cleared=" + PortSettingsArray[_port].Device);
        //#endregion Case #1 & #2b: De-instantiate previous device and set port to empty
    }
    //#endregion Case #1 & #2: New Device=PEAK_NO_DEVICE - we're clearing this Port
    else
    // Case #4 & #5: New Device=!PEAK_NO_DEVICE (new device, replacing device or modifying existing device)
    //#region Case #4 & #5: New Device=!PEAK_NO_DEVICE (new device, replacing device or modifying existing device)
    {
        // Case#4: New Device != Previous Device - first stop and remove old device, then add new device
        //#region Case#4: New Device != Previous Device - first stop and remove old device, then add new device
        if (newPortSettings.Device != PortSettingsArray[_port].Device) {

            // Case #4a: If there was a different device on this newPortSettings, stop and remove it
            //#region Case #4a: If there was a different device on this newPortSettings, stop and remove it
            if (PortSettingsArray[_port].Device !== PEAK_NO_DEVICE) {
                // Stop processing in a device was previously on this port
                PortSettingsArray[_port].DeviceObject.Enable(false);// Stop the previous device
                PortSettingsArray[_port].DeviceObject.SendEvents(false);// Stop sending events from the device
                Logger.warn("Ports.UpdatePortSettingsArrayPort: Port[" + _port + "]=" + PortSettingsArray[_port].Device + " was stopped!");
            }
            //#endregion Case #4a: If there was a different device on this newPortSettings, stop and remove it

            // Case #3 & #4b: Add the new device object instantiation to PortSettingsArray
            //#region Case #3 & #4b: Add the new device object instantiation to PortSettingsArray
            require("./devices/" + newPortSettings.Device + ".js")(_port, Logger, SocketIo, SmtpManager, NetSnmp, function (deviceObject) {
                // Update settings
                PortSettingsArray[_port].Name = newPortSettings.Name;
                PortSettingsArray[_port].Device = newPortSettings.Device;
                PortSettingsArray[_port].DeviceGroup = newPortSettings.DeviceGroup;
                PortSettingsArray[_port].Enabled = newPortSettings.Enabled;
                PortSettingsArray[_port].SendEvents = newPortSettings.SendEvents;
                PortSettingsArray[_port].TimeChanged = moment();// Device has changed
                PortSettingsArray[_port].DeviceObject = deviceObject;
                PortSettingsArray[_port].DeviceModel = PortSettingsArray[_port].DeviceObject.GetDeviceModel();// Default device model

                // Enable processing/eventing
                PortSettingsArray[_port].DeviceObject.Enable(PortSettingsArray[_port].Enabled);// Start or stop the device
                PortSettingsArray[_port].DeviceObject.SendEvents(PortSettingsArray[_port].SendEvents);// SendEvents or not from the device
                PortSettingsArray[_port].DeviceObject.SetSnmpHostSessionsArray(SnmpHostSessionsArray, function () { });// Update the SNMP Sessions Array

                Logger.warn("Ports.UpdatePortSettingsArrayPort: Port[" + _port + "] New device instantiated=" + PortSettingsArray[_port].Device + " (uuid=" + PortSettingsArray[_port].DeviceModel.uuid + ")");
            });
            //#endregion Case #3 & #4b: Add the new device object instantiation to PortSettingsArray
        }
        //#endregion Case#4: New Device != Previous Device - first stop and remove old device, then add new device

        // Case #5: New Device == Previous Device - use existing device instantiation, modify parameters
        //#region Case #5: New Device == Previous Device - use existing device instantiation, modify parameters
        else {
            // Update settings
            PortSettingsArray[_port].Name = newPortSettings.Name;
            PortSettingsArray[_port].Device = newPortSettings.Device;
            PortSettingsArray[_port].DeviceGroup = newPortSettings.DeviceGroup;
            PortSettingsArray[_port].Enabled = newPortSettings.Enabled;
            PortSettingsArray[_port].SendEvents = newPortSettings.SendEvents;
            PortSettingsArray[_port].TimeChanged = moment();// Device has changed
            // DeviceObject, DeviceModel don't change!

            // Set the operating parameters - yes, I realize that this is duplicating the code above, but just did it for clarity
            PortSettingsArray[_port].DeviceObject.Enable(PortSettingsArray[_port].Enabled);// Start or stop the device
            PortSettingsArray[_port].DeviceObject.SendEvents(PortSettingsArray[_port].SendEvents);// SendEvents or not from the device
            PortSettingsArray[_port].DeviceObject.SetSnmpHostSessionsArray(SnmpHostSessionsArray, function () { });// Update the SNMP Sessions Array
            Logger.warn("Ports.UpdatePortSettingsArrayPort: Port[" + _port + "] Updated settings for Device=" + PortSettingsArray[_port].Device + "(" + PortSettingsArray[_port].DeviceModel.uuid + ")");
        }
        //#endregion Case #5: New Device == Previous Device - use existing device instantiation, modify parameters
    }
    //#endregion Case #4 & #5: New Device=!PEAK_NO_DEVICE (new device, replacing device or modifying existing device)

    // Update ports.json to ensure persistence of the newPortSettings parameters
    //#region Update ports.json to ensure persistence of the newPortSettings parameters
    fs.truncate(PEAK_PORTS_SETTINGS, 0, function () {
        //#region Update ports.json
        fs.writeFile(PEAK_PORTS_SETTINGS, JSON.stringify(PortSettingsArray), function (error) {
            if (error) {
                Logger.error("Ports.UpdatePortSettingsArrayPort: ports.json write error=" + error);
            }
            cp.exec("sync", function (error, stdout, stderr) {
                Logger.debug("Ports.UpdatePortSettingsArrayPort: sync " + PEAK_PORTS_SETTINGS);
                if (error) {
                    Logger.error("Ports.UpdatePortSettingsArrayPort: sync error=" + error);
                } else {
                    Logger.warn("Ports.UpdatePortSettingsArrayPort: Port Settings Updated! '" + PEAK_PORTS_SETTINGS + "'");
                }
            });
            //#endregion Update ports.json

            // Update the device[newPortSettings].json
            //#region  Update the device[newPortSettings].json
            if (PortSettingsArray[_port].Device !== PEAK_NO_DEVICE) {
                fs.truncate(PEAK_DEVICE_SETTINGS + _port + ".json", 0, function () {
                    fs.writeFile(PEAK_DEVICE_SETTINGS + _port + ".json", JSON.stringify(PortSettingsArray[_port].DeviceModel), function (error) {
                        if (error) {
                            Logger.error("Ports.UpdatePortSettingsArrayPort: 'device" + _port + ".json' write error=" + error);
                        }
                        cp.exec("sync", function (error, stdout, stderr) {
                            Logger.debug("Ports.UpdatePortSettingsArrayPort: sync " + PEAK_DEVICE_SETTINGS + _port);
                            if (error) {
                                Logger.error("Ports.UpdatePortSettingsArrayPort: sync error=" + error);
                            } else {
                                Logger.warn("Ports.UpdatePortSettingsArrayPort: Device Updated! '" + PEAK_DEVICE_SETTINGS + _port + ".json" + "'");
                            }
                        });
                    });
                });
            }
            //#endregion  Update the device[newPortSettings].json
            else
            // Delete device[newPortSettings].json file
            //#region Delete device[newPortSettings].json file
            {
                // Delete previous persistence settings and log files for NULL devices
                if (fs.existsSync(PEAK_DEVICE_SETTINGS + _port + ".json")) {
                    fs.unlinkSync(PEAK_DEVICE_SETTINGS + _port + ".json");

                    // Delete previous log files for the device
                    if (fs.existsSync(PEAK_PORTS_LOGFILE + _port + "events.json")) {
                        fs.unlinkSync(PEAK_PORTS_LOGFILE + _port + "events.json");
                    }

                    // Do a sync to ensure that the files are taken care of
                    cp.exec("sync", function (error, stdout, stderr) {
                        Logger.debug("Ports.UpdatePortSettingsArrayPort: sync " + PEAK_DEVICE_SETTINGS + _port + ".json, " + PEAK_PORTS_LOGFILE + _port + "events.json");
                        if (error) {
                            Logger.error("Ports.UpdatePortSettingsArrayPort: sync error=" + error);
                        }
                    });
                }
            }
            //#endregion Delete device[newPortSettings].json file

            SocketIo.sockets.emit("Ports", { "Ports": PortSettingsArray });
            callback(PortSettingsArray);
        });
    })
    //#endregion Update ports.json to ensure persistence of the newPortSettings parameters
}

// Update Device on port
//      - Updates the DeviceModel defined for a set port
module.exports.UpdatePortSettingsArrayPortDevice = UpdatePortSettingsArrayPortDevice;
function UpdatePortSettingsArrayPortDevice(port, model, callback) {
    Logger.info("Ports.UpdatePortSettingsArrayPortDevice: Port=" + port + " model.id=" + model.id + " model.port=" + model.port + " model=" + JSON.stringify(model));

    // Update the device object with new settings
    PortSettingsArray[port].DeviceObject.UpdateDevice(port, model, function (updatedmodel) {
        Logger.trace("Ports.UpdatePortSettingsArrayPortDevice: Port=" + port + " Updated DeviceModel=" + JSON.stringify(updatedmodel));

        // Update data files to ensure persistence of port and device settings
        PortSettingsArray[port].DeviceModel = updatedmodel;

        fs.truncate(PEAK_PORTS_SETTINGS, 0, function () {

            // Updated ports.json with new PortSettingsArray values
            fs.writeFile(PEAK_PORTS_SETTINGS, JSON.stringify(PortSettingsArray), function (error) {
                if (error) {
                    Logger.error("Ports.UpdatePortSettingsArrayPortDevice: ports.json write error=" + error);
                }
                Logger.info("Ports.UpdatePortSettingsArrayPortDevice: PortSettingsArray has been updated with new device.id='" + updatedmodel.id + "' on Port " + port);
                cp.exec("sync", function (error, stdout, stderr) {
                    Logger.debug("Ports.UpdatePortSettingsArrayPortDevice: sync " + PEAK_PORTS_SETTINGS);
                    if (error) {
                        Logger.error("Ports.UpdatePortSettingsArrayPortDevice: sync error=" + error);
                    } else {
                        Logger.warn("Ports.UpdatePortSettingsArrayPortDevice: Port Settings Updated! '" + PEAK_PORTS_SETTINGS + "'");
                    }
                });

                // Update the device[port].json to ensure persistence of the device parameters
                fs.truncate(PEAK_DEVICE_SETTINGS + port + ".json", 0, function () {
                    fs.writeFile(PEAK_DEVICE_SETTINGS + port + ".json", JSON.stringify(PortSettingsArray[port].DeviceModel), function (error) {
                        if (error) {
                            Logger.error("Ports.UpdatePortSettingsArrayPortDevice: 'device" + port + ".json' write error=" + error);
                        }
                        Logger.info("Ports.UpdatePortSettingsArrayPortDevice: 'device" + port + ".json' has been updated with new parameters=" + JSON.stringify(PortSettingsArray[port].DeviceModel));
                        cp.exec("sync", function (error, stdout, stderr) {
                            Logger.debug("Ports.UpdatePortSettingsArrayPortDevice: sync " + PEAK_DEVICE_SETTINGS + port + ".json");
                            if (error) {
                                Logger.error("Ports.UpdatePortSettingsArrayPortDevice: sync error=" + error);
                            } else {
                                Logger.warn("Ports.UpdatePortSettingsArrayPortDevice: Device Updated! '" + PEAK_DEVICE_SETTINGS + port + ".json" + "'");
                            }
                        });
                    });
                });

                SocketIo.sockets.emit("Ports", { "Ports": PortSettingsArray });
            });
        });

        // Return the updated model
        callback(updatedmodel);
    });
}

module.exports.GetPortSettingsArray = GetPortSettingsArray;
function GetPortSettingsArray() {
    return PortSettingsArray;
}

module.exports.GetDeviceInformation = GetDeviceInformation;
function GetDeviceInformation(port, callback) {
    // Update the device object with new settings
    PortSettingsArray[port].DeviceObject.GetDeviceInformation(function (deviceInfo) {
        callback(deviceInfo);
    });
}

//#endregion PortSettingsArray functions

//#endregion Functions